/**
*   Nombre del Programa: Caminante_DATOS
*   Proyecto: Aplicacin Android de Asistencia al Caminante
*   Uso: captura coordenadas de sensores (acelerometro, orientacin)
    	 pasandolas a ficheros de texto	para su estudio.
**/    
/**    
    Copyright (C) 2011 Enrique Ramn Delgado Garrido

    This program is free software: you can redistribute it and/or modify 
    it under the terms of the GNU General Public License as published by 
    the Free Software Foundation, either version 3 of the License, or 
    (at your option) any later version. 

    This program is distributed in the hope that it will be useful, 
    but WITHOUT ANY WARRANTY; without even the implied warranty of 
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
    GNU General Public License for more details. 

    You should have received a copy of the GNU General Public License 
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
**/


package com.caminante.datos;

import java.io.BufferedWriter;

import java.io.File;
//import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;

import java.sql.Timestamp;

import java.util.ArrayList;
import java.util.Calendar;

//import android.app.Activity;
import android.content.ActivityNotFoundException;
import android.content.Context;
import android.content.Intent;
import android.net.Uri;
import android.os.Environment;
import android.util.Log;

/** Esta clase contiene los elementos que tienen relacion con los ficheros 
 *  de texto en que se almacenan los datos capturados por los sensores.
 *  Tambin se utiliza para la lanzar el correo en que dichos ficheros se envian a una cuenta
 *  
 * 
 * @author Enrique
 *
 */
public class FicherosCAMINANTE { 
/** Variables para el uso del nombre del caminante **/
	static	String 				nameDirectory 	= "Caminar_";
	static 	String				walker			= "";	
/** Variables para el trataminto de los ficheros y su escritura **/
	private	BufferedWriter		bufferedWriterACEL;
	private File 				txtFileACEL			=	null;
	private	BufferedWriter		bufferedWriterORIE;
	private File 				txtFileORIE			=	null;

	private String 				headerLine;
	private String 				dataLine;
	private Long 				timeStamp		= 	new Timestamp(Calendar.getInstance().getTimeInMillis()).getTime();		
	private String 				nameFileACEL 	= 	"ACEL_" + timeStamp.toString() + ".txt";
	private String 				nameFileORIE 	= 	"ORIE_" + timeStamp.toString() + ".txt";
	private String 				mailFileACEL 	= 	"";
	private String 				mailFileORIE 	= 	"";
/** Variables para la localizacin de los ficheros y su nombre **/
	private static final 	String EXT_PATH 		= Environment.getExternalStorageDirectory()+"/Android/data/com.caminante.datos/";
	private static 			String EXT_PATH_walker 	= null;

/** Constructor -ficticio- para paso del nombre del caminante
 *  se aprovecha para la creacin del nombre del directorio de almacenamiento
 *  de los ficheros. Puede obviarse sustituyendolo por otras opciones 
 *  para el paso del nombre  
 *
 */

	public FicherosCAMINANTE(String wALKER, Integer nUM) {
		nameDirectory = "Caminar_" + wALKER;
		walker = wALKER;
	}
/** Constructor -real- 
 * 
 */
	public FicherosCAMINANTE(Context ctx){

		EXT_PATH_walker = EXT_PATH + nameDirectory;

		try {
			if (Environment.getExternalStorageState().equals(Environment.MEDIA_MOUNTED)) {
				// creacin del directorio raiz si no existe
				new File(EXT_PATH).mkdirs();
				// creacin del directorio para el nombre del caminante
				new File(EXT_PATH_walker).mkdirs();
			} else {
				File dir = new File(EXT_PATH_walker);
				if (!dir.exists()) {
					// creacin del directorio raiz porqu no existe
					new File(EXT_PATH_walker).mkdirs();
				}
			}
			//llamada que crea los ficheros de texto si no existen
			FicheroACELEROMETRO(ctx);
			FicheroORIENTACION(ctx);
 		} catch (Exception e) {
	        Log.w("ExternalStorage", "ErrorDIRECTORY" + e.getMessage());
		}
	}
/**
 * 
 * @param ctx
 */
	protected void FicheroACELEROMETRO(Context ctx) {
 		
		try {
			// crea el fichero si no existe y lo asigna si ya existe
			txtFileACEL = new File(EXT_PATH_walker, nameFileACEL);
			if (!txtFileACEL.exists()) {
				txtFileACEL.createNewFile();
			} else {
				txtFileACEL = ctx.getFileStreamPath(nameFileACEL);
			}
			// escibe la linea de cabecera
			bufferedWriterACEL = new BufferedWriter(new OutputStreamWriter( new FileOutputStream(txtFileACEL))); 
			headerLine = String.format("%s\t%s\t%s\t%s\n",
										  "coorAcel_0","coorAcel_1","coorAcel_2",
										  "tiempo");		
 			bufferedWriterACEL.write(headerLine);
 			bufferedWriterACEL.flush();
 		} catch (IOException ioe) {
	        Log.w("ExternalStorage", "Error writing" + nameFileACEL, ioe);
		} finally {
    		try {
    			if (bufferedWriterACEL == null) {
    				bufferedWriterACEL.close();
    			}
    		} catch (Exception e) {
                Log.d("ExceptionACEL: ", "finally: " + e.getMessage());
    		}
		}
	}
/**
 * 
 * @param ctx
 */
	protected void FicheroORIENTACION(Context ctx) {
 		
		try {
			// crea el fichero si no existe y lo asigna si ya existe
			txtFileORIE = new File(EXT_PATH_walker, nameFileORIE);
			if (!txtFileORIE.exists()) {
				txtFileORIE.createNewFile();
			} else {
				txtFileORIE = ctx.getFileStreamPath(nameFileORIE);
			}			
			// escibe la linea de cabecera
			bufferedWriterORIE = new BufferedWriter(new OutputStreamWriter( new FileOutputStream(txtFileORIE))); 
			headerLine = String.format("%s\t%s\t%s\t%s\n",
										  "coorMagn_0","coorMagn_1","coorMagn_2",
										  "tiempo");		
 			bufferedWriterORIE.write(headerLine);
 			bufferedWriterORIE.flush();
 		} catch (IOException ioe) {
	        Log.w("ExternalStorage", "Error writing" + nameFileORIE, ioe);
		} finally {
    		try {
    			if (bufferedWriterORIE == null) {
    				bufferedWriterORIE.close();
    			}
    		} catch (Exception e) {
                Log.d("ExceptionORIE: ", "finally: " + e.getMessage());
    		}
		}
	}
/** Mdulo que escribe los datos del acelermetro en su fichero 
 * 
 * @param ca0
 * @param ca1
 * @param ca2
 */
	public void addDatoACEL(float ca0, float ca1, float ca2){
 		try{
 			// captura del valor de TimeStamp, se hace aqui para pasar
 			// un parametro menos
 			Long timeStamp = new Timestamp(Calendar.getInstance().getTimeInMillis()).getTime();	
 			// dar formato a la linea de datos y escribirlos
 			dataLine = String.format("%.10f\t%.10f\t%.10f\t%d\n", 
 									    ca0,   ca1,   ca2,   timeStamp);
 			bufferedWriterACEL.write(dataLine);
 			bufferedWriterACEL.flush();
 		} catch (Exception e) {
            Log.d("Exception: ", "escribirACEL: " + e.getMessage());
        } finally {
    		try {
    			if (bufferedWriterACEL == null) {
    				bufferedWriterACEL.close();
    			}
    		} catch (Exception e) {
                Log.d("Exception: ", "finallyACEL: " + e.getMessage());
    		}
        }
 	}
/** Mdulo que escribe los datos de orientacin en su fichero
 *  
 * @param cm0
 * @param cm1
 * @param cm2
 */
	public void addDatoORIE(float cm0, float cm1, float cm2){
 		try{
 			// captura del valor de TimeStamp, se hace aqui para pasar
 			// un parametro menos
			Long timeStamp = new Timestamp(Calendar.getInstance().getTimeInMillis()).getTime();	
			// dar formato a la linea de datos y escribirlos
			dataLine = String.format("%.10f\t%.10f\t%.10f\t%d\n", 
 									  cm0,   cm1,   cm2,   timeStamp);
 			bufferedWriterORIE.write(dataLine);
 			bufferedWriterORIE.flush();
 		} catch (Exception e) {
            Log.d("Exception: ", "escribirORIE: " + e.getMessage());
        } finally {
    		try {
    			if (bufferedWriterORIE == null) {
    				bufferedWriterORIE.close();
    			}
    		} catch (Exception e) {
                Log.d("Exception: ", "finallyORIE: " + e.getMessage());
    		}
        }
 	}
/** mdulo para cerrar los ficheros y mandar el correo con los ficheros
 *  adjuntos
 * @param ctx
 */
	public void fileClose(Context ctx){
		mailFileACEL = EXT_PATH_walker + "/" + nameFileACEL;
		mailFileORIE = EXT_PATH_walker + "/" + nameFileORIE;
		try {
			bufferedWriterACEL.close();
		} catch (IOException ioe) {
			// TODO Auto-generated catch block
			mailFileACEL = "/";
			Log.i("IOException: ", "cerrar ACEL" + ioe.getMessage());
		} finally {
    		try {
    			if (bufferedWriterACEL == null) {
    				bufferedWriterACEL.close();
    			}
    		} catch (IOException ioe) {
                Log.d("IOException: ", "finally ACEL: " + ioe.getMessage());
        		mailFileACEL = "/";
    		}
    	}
		try {
			bufferedWriterORIE.close();
		} catch (IOException ioe) {
			// TODO Auto-generated catch block
			mailFileORIE = "/";
			Log.d("IOException: ", "close ORIE" + ioe.getMessage());
		} finally {
    		try {
    			if (bufferedWriterORIE == null) {
    				bufferedWriterORIE.close();
    			}
    		} catch (IOException ioe) {
    			mailFileORIE = "/";
                Log.d("IOException: ", "finally ORIE: " + ioe.getMessage());
    		}
    	}
		// lanzando mdulo para enviar el correo
		sendFile(ctx);
	}
/** mdulo para enviar el correo
 * 
 * @param ctx
 */
	public void sendFile(Context ctx){		
		// creacin y carga del Intent
		Intent emailIntent = new Intent(android.content.Intent.ACTION_SEND_MULTIPLE); 
		String[] recipients = new String[]{"walkercaminante@gmail.com", "",}; 
		emailIntent.putExtra(android.content.Intent.EXTRA_EMAIL, recipients); 
		emailIntent.putExtra(android.content.Intent.EXTRA_SUBJECT, "Datos de " + walker); 
		
		if ((mailFileACEL.equals("/")) & (mailFileORIE.equals("/"))) {
			emailIntent.putExtra(android.content.Intent.EXTRA_TEXT, "NO se crearon ficheros");
		} else {			 
			String cseq = String.format("%s\n%s\n%s\n%s\n%s\n%s\n%s\n", 
							     		"Informacin complementaria a los datos",
							     		"  Posicin del mvil",
							     		"  Orientacin del mvil",
							     		"  Bloqueo",
							     		"  Parada",
							     		"  Cada",
							     		"  Tropiezo"); 
			emailIntent.putExtra(android.content.Intent.EXTRA_TEXT,  cseq.toString()); 

			emailIntent.setType("text/plain");				
			ArrayList<Uri> uris = new ArrayList<Uri>();
			if (mailFileACEL!=("/")) {
				Uri uACEL = Uri.parse ( "file://" + mailFileACEL);
				uris.add(uACEL);		
			}
			if (mailFileORIE!=("/")) {
				Uri uORIE = Uri.parse ( "file://" + mailFileORIE);
				uris.add(uORIE);					
			}
			emailIntent.addFlags(android.content.Intent.FLAG_GRANT_READ_URI_PERMISSION);
			emailIntent.putParcelableArrayListExtra(Intent.EXTRA_STREAM, uris);			
		}
		try {
			ctx.startActivity(Intent.createChooser(emailIntent, "Send mail..."));
		} catch  (ActivityNotFoundException eanf) {
			Log.w("ActivityException", eanf.getMessage());
		} 
	}
}